import { Component, createEffect, createSignal, onMount } from 'solid-js';
import {
  useParams,
  useNavigate,
  useRouteData,
  useBeforeLeave,
} from '@solidjs/router';
import OlMap from 'ol/Map';
import View from 'ol/View';
import TileLayer from 'ol/layer/Tile';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import Feature from 'ol/Feature';
import Attribution from 'ol/control/Attribution';
import Rotate from 'ol/control/Rotate';
import ScaleLine from 'ol/control/ScaleLine';
import Control from 'ol/control/Control';
import OSM from 'ol/source/OSM';
import { useGeographic as olUseGeographic } from 'ol/proj';

import 'ol/ol.css';
import './Map.css';
import { Collection } from 'ol';
import GetLocation from '../get-location';

const [getState, setState] = createSignal({
  lon: 0,
  lat: 0,
  rotation: 0,
  zoom: 0,
});

export { getState };

const Map: Component = () => {
  const [getMap, setMap] = createSignal<OlMap | null>(null);
  const params = useParams();
  const navigate = useNavigate();

  let target: HTMLDivElement;
  console.log({ caller: 'Map', center: params.center });


  createEffect(() => {
    setState({
      lon: +params.lon,
      lat: +params.lat,
      rotation: +params.rotation,
      zoom: +params.zoom,
    });
    const map = getMap();
    const view = map?.getView();
    view?.animate({
      center: [getState().lon, getState().lat],
      rotation: getState().rotation,
      zoom: getState().zoom,
      duration: 1000,
    });
  });

  onMount(() => {
    olUseGeographic();

    const changeListener = (event: any) => {
      const map = getMap();
      const view = map?.getView();
      const center = view?.getCenter();
      if (center) {
        navigate(
          `/map/${center[0]}/${
            center[1]
          }/${view?.getZoom()}/${view?.getRotation()}`
        );
      }
      console.log({
        caller: 'Map / changeListener',
        event,
        params,
        map,
      });
    };

    const olMap = new OlMap({
      view: new View({
        center: [+getState().lon, +getState().lat],
        zoom: +getState().zoom,
        rotation: +getState().rotation,
      }),
      layers: [
        new TileLayer({
          source: new OSM(),
        }),
        //new VectorLayer({ source: newSource }),
      ],
      target: target,
      controls: new Collection<Control>([
        new Attribution({ collapsible: true }),
        new Rotate(),
        new ScaleLine({ bar: true }),
        // @ts-ignore
        new Control({
          // @ts-ignore
          element: (
            <div>
              <GetLocation />
            </div>
          ),
        }),
      ]),
    });
    olMap.on(['moveend'], changeListener);

    setMap(olMap);
  });

  // @ts-ignore
  return <div class='ol-map' ref={target} />;
};

export default Map;
